## RANDOM
## This is the main program: it implements  static analysis.

## Copyright 2008-2012 Gianluca Amato, Francesca Scozzari

## This file is part of RANDOM: R-based Analyzer for Numerical DOMains
##
## RANDOM is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## RANDOM is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty ofa
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with RANDOM.  If not, see <http://www.gnu.org/licenses/>.

## Elenca gli identificatori che appaiono in una call, al di fuori di una operazione
## di subsetting e return

vars.call =  function(expr) {
  if (is.symbol(expr))
    as.character(expr)
  else if (is.atomic(expr))
    as.character()
  else if (expr[[1]] == "[")
    as.character()
  else if (expr[[1]] == "return")
    as.character()
  else {
    vars = as.character()
    if (length(expr)>=2)
      for (i in 2:length(expr)) {
        vars = union(vars, vars.call(expr[[i]]))
      }
    vars      
  }
}         

## Costruttore per una condition di classe "analyzerCondition"

analyzerCondition <- function(message) {
  cond <- simpleCondition(message)
  class(cond) <- c("analyzerCondition",class(cond))
  cond
}

## Lookout for the tag tagname in the expression expr. It returns the number
## of the tag if expr is a composed instruction whose first call is a tagname.

lookout.tag <- function(expr, tagname) {
	if (is.symbol(expr) || is.atomic(expr)) 
		NULL
	else if (expr[[1]]=="{" && length(expr)>=2) 
		lookout.tag(expr[[2]])
	else if (expr[[1]]==tagname && length(expr)>=2)
		expr[[2]]
	else 
		NULL
}
 
## Analizza una funziona fun, dopo averla taggata per identificare i vari punti
## del codice sorgente.

analyze.function <- function (fun, domain, vars, absenv, debug.analysis=FALSE, startzero=c("local","no","all"),displayed.forms=character()) {
  currpoint <- 0L
  replace.tag <- function(expr) {   
    if (is.symbol(expr) || is.atomic(expr))
      expr
    else if (expr[[1]]==".savetag") {
      output <- paste("[ ",domain$as.character(savedata[[expr[[2]]]],vars)," ]",sep="")
      if (length(displayed.forms)>0)
        for (i in 1:length(displayed.forms)) {
          lf <- as.linear.form(parse(text = displayed.forms[i])[[1]],vars)
          output <- paste(output,domain$outputform(savedata[[expr[[2]]]],lf,vars))
	    }     
      output    
    }
    else {
      tmplist <- vector("list",length(expr))
      tmplist[[1]] <- as.character(expr[[1]])
	  if (length(expr)>1)
        for (i in 2:length(expr))
          tmplist[[i]] <- replace.tag(expr[[i]])
      do.call("call",tmplist,quote=TRUE)
    }
  }
  
  if (missing(vars)) 
	  vars <- vars.call(body(fun))
  if (missing(absenv)) {
    startzero <- match.arg(startzero)
    switch (startzero,
      "no" = { absenv = domain$full(length(vars)) },
      "all" = { absenv = domain$zero(length(vars)) },
      "local" = {
        boxdom = box.domain()
        absenv = boxdom$full(length(vars))
        parameter.names = names(formals(fun))
        for (i in 1:length(vars))
          if (! (vars[i] %in% parameter.names)) {
            absenv = boxdom$select(absenv,c(as.integer(vars[i]==vars),0))
            absenv = boxdom$select(absenv,-c(as.integer(vars[i]==vars),0))
          }
        absenv=domain$frombox(absenv)
      },
      invokRestart("invalidinput")
    )
  }
  newf <- annotate(fun, function() { currpoint <<- currpoint+1; bquote(.savetag(.(currpoint))) }, skip=c(".tracetag"))
  
  savedata <- vector("list",currpoint)
  environment(analyze.call) <- sys.frame(sys.nframe())
  environment(analyze.condition) <- sys.frame(sys.nframe())

  analyze.call(body(newf), domain, vars, absenv)
  environment(analyze.call) <- .GlobalEnv
  environment(analyze.condition) <- .GlobalEnv

  res <- list( out = replace.tag(body(newf)), annotated = body(newf), domain = domain, vars = vars, program.points = savedata )
  class(res) <- "staticanalysis"
  res
}

## Procedura di stampa per la classe S3 "staticanalysis"

print.staticanalysis = function (analysis) { 
  print(analysis$out)
}

## Analizza una call (eventualmente taggata). newparam is a suggestion on what
## parameters provide later

analyze.call <- function (expr, domain, vars, data, newparam=NULL) { 
  ## C'era una volta uno shortcut per il caso data sia vuoto... adesso
  ## l'ho tolto ma non sono sicuro che funzioni. Non supporta assegnamenti
  ## a vettori.
  ##
  ## if (domain$is.empty(data)) return(data)

  switch (as.character(expr[[1]]),
    ".savetag" = {
	  if (exists("savedata")) savedata[[expr[[2]]]] <<- data
      data      
    },
	".tracetag" = {
		if (length(expr)>1)	{
			point <- expr[[2]]
			if (is.numeric(point) && point>=1)
      			domain$setparam(data, expr[[2]])
			else
				signalCondition(analyzerCondition("The argument of .tracetag should be a natural number"))
		} else
			domain$setparam(data, NULL)
	},
	"assume" = {
      absenvif <- analyze.condition(expr[[2]], domain, vars, data)
      absenvif$abs.then
      ##index = which(vars==expr[[2]])
      ##if (length(index)>0) 
      ##  domain$assign.interval(data, index, expr[[3]], expr[[4]])
      ##else  
      ## data
    },
    "=" = {
      if (is.symbol(expr[[2]])) {
        index <- which(vars==expr[[2]])
        if (length(index)>0) 
          tryCatch(
                   { 
                     linear.form <- as.linear.form(expr[[3]],vars)
                     domain$assign(data, index, linear.form, newparam)
                   },
                   analyzerCondition = function(c) domain$assign.unknown(data,index)
                   )
        else
          data  
      } else 
        data
    },
	"{" = {
		for (i in 2:length(expr)) {				
			if (i < length(expr)) 
				data <- analyze.call(expr[[i]], domain, vars, data, lookout.tag(expr[[i+1]],".tracetag"))
			else
				data <- analyze.call(expr[[i]], domain, vars, data, NULL)
		}     
		data
	},   
	"if"  = {		
		absenvif <- analyze.condition(expr[[2]], domain, vars, data)  
		absenv1 <- analyze.call(expr[[3]],domain,vars,absenvif$abs.then, newparam)
		if (length(expr)==4) 
			absenv2 <- analyze.call(expr[[4]],domain,vars,absenvif$abs.else, newparam)
		else
			absenv2 <- domain$setparam(absenvif$abs.else, newparam)
		domain$union(absenv1,absenv2)
	},
    "while" = {
      invariant <- data
      steps <- 1
	  delay <- 2
      if (debug.analysis) {
        cat("WHILE DEBUGGING\n")
        cat("INPUT:",domain$as.character(data,vars),"\n")
      }
      repeat {
        old.invariant <- invariant
        absenv.condition <- analyze.condition(expr[[2]], domain, vars, old.invariant)
        absenv.body <- analyze.call(expr[[3]],domain,vars,absenv.condition$abs.then)
		invariant <- domain$widening(old.invariant, absenv.body, delay)
        if (debug.analysis) {
          cat("ASCENDING STEP: ",steps,"\n")
          print(replace.tag(expr))
          cat("OLD INVARIANT: ", domain$as.character(old.invariant, vars),
			  "\nCANDIDATE INVARIANT:",domain$as.character(absenv.body,vars),
              "\nWIDENING: ",domain$as.character(invariant,vars),"\n") 
        }
        steps <- steps+1
		if (delay>0) delay <- delay-1
        if (domain$is.equal(invariant,old.invariant)) break
      }
      repeat {
        old.invariant <- invariant
        absenv.condition <- analyze.condition(expr[[2]], domain, vars, old.invariant)
        absenv.body <- analyze.call(expr[[3]],domain,vars,absenv.condition$abs.then)
        invariant <- domain$narrowing(old.invariant, absenv.body)
        if (debug.analysis) {
          cat("DESCENDING STEP: ",steps,"\n")
          print(replace.tag(expr))
          cat("OLD INVARIANT: ", domain$as.character(old.invariant, vars),
			  "\nCANDIDATE INVARIANT:",domain$as.character(absenv.body,vars),
              "\nNARROWING: ",domain$as.character(invariant,vars),"\n") 
        }
        steps <- steps +1
        if (domain$is.equal(invariant,old.invariant)) break
      }
      if (debug.analysis) {
        cat("END WHILE\n\n")
      }
      domain$intersect(absenv.condition$abs.else,invariant)
    },
    "return" = {
      domain$empty(length(vars))
    },
	{
		signalCondition(analyzerCondition(paste("Not supported operation :",expr[[1]])))
	}
  )
}

## Analizza la condizione (anche non atomica) expr

analyze.condition = function(expr,domain,vars,data) {
  if (expr == TRUE)
    list(abs.then = data, abs.else=domain$empty(domain$getsize(data)))
  else if (expr == FALSE)
    list(abs.then = domain$empty(domain$getsize(data)), abs.else= data)
  else {
    operator <- as.character(expr[[1]])
    switch(operator,	 	
         "{" = {
           for (i in 2:length(expr)) {
			   result <- analyze.condition(expr[[i]],domain,vars,data)
			   if (class(expr[[i]])=="call" && expr[[i]][[1]]==".tracetag" && length(expr[[i]])>1) 
				   data <- domain$setparam(data,expr[[i]][[2]])				   			   				   			   
		   }             
           result
         },
         ".savetag" = {
           if (exists("savedata")) savedata[[expr[[2]]]] <<- data
         },
		 ".tracetag" = {
		 },
         "&&" = {
           abs1 = analyze.condition.atomic(expr[[2]],domain,vars,data)
           abs2 = analyze.condition.atomic(expr[[3]],domain,vars,data)
           list (
                 abs.then = domain$intersect(abs1$abs.then, abs2$abs.then),
                 abs.else = domain$union(abs1$abs.else, abs2$abs.else)
                 )
         },
         "||" = {
           abs1 = analyze.condition.atomic(expr[[2]],domain,vars,data)
           abs2 = analyze.condition.atomic(expr[[3]],domain,vars,data)
           list (
                 abs.then = domain$union(abs1$abs.then, abs2$abs.then),
                 abs.else = domain$intersect(abs2$abs.else, abs2$abs.else)
                 )
         },
         "!" = {
           abs1 = analyze.condition.atomic(expr[[2]],domain,vars,data)
           list (
                 abs.then = abs1$abs.else,
                 abs.else = abs1$abs.then
                 )
         },       
		 "brandom" = {			 
			 list(abs.then=data, abs.else = data)
		 },
         analyze.condition.atomic(expr,domain,vars,data)
         )
  }
}

## Analizza la condizione atomica expr, e restituisce gli ambienti astratti per i
## casi then ed else. Assume che i tipi di dato siano reali

analyze.condition.atomic = function (expr, domain, vars, data) {
  tryCatch( 
    {
      linear.form = as.linear.form(expr[[2]],vars) - as.linear.form(expr[[3]],vars)
      leq = domain$select(data, linear.form)
	  geq = domain$select(data, -linear.form)
	  diff = domain$select.diff(data, linear.form)
	  
      ## volevo usare switch.. ma qualche bug perché va in loop
      operator =  as.character(expr[[1]])
      if (operator=="<=") 
        list ( 
              abs.then = leq, 
              abs.else = domain$intersect(geq,diff)
              )
      else if (operator==">=")
        list ( 
              abs.then = geq, 
              abs.else = domain$intersect(leq,diff)
              )
      else if (operator=="==") 
        list ( 
              abs.then = domain$intersect(leq,geq),
              abs.else = domain$select.diff(data, linear.form)
              )
      else if (operator=="!=") 
        list (
              abs.then = domain$select.diff(data, linear.form),
              abs.else = domain$intersect(leq,geq)
              )
      else if (operator=="<")
        list (
              abs.then = domain$intersect(leq,diff),
              abs.else = geq
              )
      else if (operator==">")
        list (
              abs.then = domain$intersect(geq,diff),
              abs.else = leq
              )
      else
        signalCondition(analyzerCondition("Unknown relational operator"))
    },
    analyzerCondition = function(c) { 
      list ( abs.then = data, abs.else = data )
    }
  )
}

# Converte la call "expr" in un vettore, usando le variabili vars. Se la
# conversione non è possibile, segnala una condizione di errore.

as.linear.form = function (expr, vars) {
  if (is.numeric(expr)) {
    c(rep(0,length(vars)), expr )
  } else if (is.symbol(expr)) {
    res = c(as.numeric(vars==as.character(expr)),0)
    if (any(res!=0))
      res
    else
      signalCondition(analyzerCondition("Using extra variable"))
  } else switch(as.character(expr[[1]]), 
    "+" = { 
      as.linear.form(expr[[2]],vars) + as.linear.form(expr[[3]],vars) 
    },
    "-" = {
      if (length(expr)==2)
        - as.linear.form(expr[[2]],vars)
      else
        as.linear.form(expr[[2]],vars) - as.linear.form(expr[[3]],vars) 
    },    
    "*" = {  
      coeff.index = length(vars)+1
      exp1 = as.linear.form(expr[[2]],vars)  
      exp2 = as.linear.form(expr[[3]],vars)
      if (all(exp1[-coeff.index]==0))
        exp2 * exp1[coeff.index]
      else if (all(exp2[-coeff.index]==0))
        exp1 * exp2[coeff.index]
      else
        signalCondition(analyzerCondition("Not a linear.term"))
    },
    "%/%" = {
      coeff.index = length(vars)+1
      exp1 = as.linear.form(expr[[2]],vars) 
      exp2 = as.linear.form(expr[[3]],vars)
      if (all(exp2[-coeff.index]==0)) 
        exp1 / exp2[coeff.index]
      else 
        signalCondition(analyzerCondition("Not a linear term"))
    },
   "/" = {
      coeff.index = length(vars)+1
      exp1 = as.linear.form(expr[[2]],vars) 
      exp2 = as.linear.form(expr[[3]],vars)
      if (all(exp2[-coeff.index]==0)) 
        exp1 / exp2[coeff.index]
      else
        signalCondition(analyzerCondition("Not a linear term"))
    },                
    "(" = as.linear.form(expr[[2]],vars),
    signalCondition(analyzerCondition("Unrecognized operator"))
  )
}


## Print numbers with a given number of digits and right rounding

num.to.char <- function(n, direction = c("lower", "upper"), digits=3)  {
	if (! is.finite(n)) 
		as.character(n)
	else {
		frac <- abs(n - trunc(n)) * 10^digits
		if (direction=="upper" && trunc(frac) != frac) frac = frac+1		
		if (trunc(frac) != 0)
			paste(as.character(trunc(n)), ".", trunc(frac), sep="")
		else 
			as.character(trunc(n))
	}    
}

num.to.char.low <- function(n,...) num.to.char(n,"lower",...)
num.to.char.high <- function(n,...) num.to.char(n,"upper",...)

# Display linear forms

linear.form.display=function (linear.form, vars, digits=NULL) {

	monom.display = function(coeff, var) {
    if (coeff==0)
      ""
    else if (var=="")
      as.character(coeff)
    else if (coeff==1)
      as.character(var)
    else if (coeff==-1)
      paste("-",var,sep="")
    else if (is.null(digits))
      paste(coeff,"*",var,sep="")
  	else 
	  paste(round(coeff,digits),"*",var,sep="")
  }

  first=TRUE
  out=""
  vars[length(linear.form)]=""
  for (i in 1:length(linear.form)) {
    if (linear.form[i]>0) {
      if (first) 
        out=paste(out,monom.display(abs(linear.form[i]),vars[i]),sep="")
      else
        out=paste(out,"+",monom.display(abs(linear.form[i]),vars[i]),sep="")
      first=FALSE
    } else if (linear.form[i]<0) {
      out=paste(out,"-",monom.display(abs(linear.form[i]),vars[i]),sep="")
      first=FALSE
    }
  }
  out
}

#### Compare results

compare.analyses = function(f,params=list(),vars,inverse=TRUE,orthogonal=TRUE,theta=pi/4,only.while=FALSE,...) {
	if (length(params)==0 || mode(params[[1]])!="list") params=list(params)
	
	time = Sys.time()
	if (!missing(vars)) 
		collect.trace(f,params,vars=vars,only.while=only.while)
	#trace = bind.traces(lapply(params,function(p) collect.trace(f,p,vars=vars,only.while=only.while)))
	else
		collect.trace(f,params,only.while=only.while)
	#trace = bind.traces(lapply(params,function(p) collect.trace(f,p,only.while=only.while)))
	cat(paste("Collect traces:",(Sys.time()-time)," seconds\n"))
	
	time = Sys.time()
	pca = analyze.trace(trace)
	cat(paste("Analyze traces:",(Sys.time()-time)," seconds\n"))
	
	m = pca$rotation
	if (missing(vars)) vars = rownames(m)      
	if (inverse) {
		m1=t(simplify(m,theta,orthogonal=orthogonal))
	} else {	
		m1=simplify(m,theta,orthogonal=orthogonal)  
	}
	
	time=Sys.time()
	res1 = analyze.function(f,box.domain(),startzero="local",vars=vars,...)
	cat(paste("Box analyses:",-(time-Sys.time())," seconds\n"))
	
	#time=Sys.time()
	#res1gmp = analyze.function(f,box.domain.gmp(),startzero="local",vars=vars,...)
	#cat(paste("Box GMP analyses:",-(time-Sys.time())," seconds\n"))
	
	time=Sys.time()
	res2 = analyze.function(f,rotatedbox.domain(m1,inverse),startzero="local",vars=vars,...)
	cat(paste("Paralleolotopes analyses:",-(time-Sys.time())," seconds\n"))
	
	#time=Sys.time()
	#res2gmp = analyze.function(f,rotatedbox.domain.gmp(m1,inverse),startzero="local",vars=vars,...)
	#cat(paste("Paralleolotopes GMP analyses:",-(time-Sys.time())," seconds\n"))
	
	time=Sys.time()
	res3 = analyze.function(f,combinedbox.domain(m1,inverse),startzero="local",vars=vars,...)
	cat(paste("Combined analyses:",-(time-Sys.time())," seconds\n"))
	
	#time=Sys.time()
	#res3gmp = analyze.function(f,combinedbox.domain.gmp(m1,inverse),startzero="local",vars=vars,...)
	#cat(paste("Combined analyses:",-(time-Sys.time())," seconds\n"))
	
	list(m,m1,box=res1,boxgmp=null, ptopes=res2, ptopesgmp=null, combo=res3, combogmp=null)  
}

## Dichirazione delle funzione dummy assume e brandom
assume = function(...) {
}

brandom = function() {
	rbinom(1,1,c(0.5,0.5)) == 1
}

