## RANDOM
## This file contains the annotate functions: they annotate the code of a program with special
## instructions.

## Copyright 2008-2011 Gianluca Amato, Francesca Scozzari

## This file is part of RANDOM: R-based Analyzer for Numerical DOMains
##
## RANDOM is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## RANDOM is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty ofa
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License

# This is the generic function for annotate

setGeneric("annotate", function(object, ...) standardGeneric("annotate"))

# Annotate for the language class takes a call (expr) and three calls (between,  start, end). 
# It annotates expr by inserting the call "between" at program points, "start" at the beggining 
# of expr and "end" at the end. If only.while is TRUE, "between" is inserted only within 
# the guard of while expression. The parameter "between" may be a closure: in such a case, 
# it is called at every annotation point, and its result is injected into expr.
# The parameter skip says to not consider program points before a call to functions in skip

setMethod("annotate", c("language"), function(object, between, start, end, only.while = FALSE, skip=c()) {	
			
			annotate.call.aux <- function(expr, single.annotate) {					
				if (single.annotate && !only.while && class(expr)!="{" && !(class(expr) %in% skip))
					call("{", injection(), annotate.call.aux(expr, FALSE), injection())
				else switch(class(expr), 
							"{" = { 		
								size <- length(expr)
								if (only.while) {
									tmplist <- vector("list",size)
									tmplist[[1]] <- "{"
									for (i in 2:size)
										tmplist[[i]] = annotate.call.aux(expr[[i]], FALSE)
								} else  {
									tmplist <- list("{")
									nextpos <- 2									
									for (i in 2:size) {
										if (class(expr[[i]])!="call" || !(as.character(expr[[i]][[1]]) %in% skip)) {
											tmplist[[nextpos]] <- injection()
											nextpos <- nextpos+1
										}					
										tmplist[[nextpos]]<- annotate.call.aux(expr[[i]], FALSE)
										nextpos <- nextpos+1
									}
									tmplist[[nextpos]] <- injection()										
								}
								do.call("call", tmplist, quote = TRUE)
							},
							"while" = call("while", call ("{",injection(),expr[[2]]), annotate.call.aux(expr[[3]], TRUE)),
							"repeat" = call("repeat", annotate.call.aux(expr[[2]], TRUE)),
							"for" = call("for", expr[[2]], expr[[3]], injection()),
							"if" = if (only.while)
										expr
									else if (length(expr)==4)
										call("if", expr[[2]], annotate.call.aux(expr[[3]],TRUE), annotate.call.aux(expr[[4]],TRUE))
									else
										call("if", expr[[2]], annotate.call.aux(expr[[3]],TRUE)),
							expr
					)
			}
			injection <- switch(mode(between),
					"call" = function() { between },
					"function" = between,
					stop("Invalid mode for between, it should be a call or a function")
			)
			
			annotated.call <- annotate.call.aux(object, TRUE)
			size <- length(annotated.call)
			incr.for.end <- 1
			if (!missing(start)) {
				incr.for.end <- 2
				annotated.call[2:size + 1] <- annotated.call[ 2:size ]
				annotated.call[[2]] <- start
			}
			if (!missing(end))
				annotated.call[[size+incr.for.end]] <- end
			annotated.call
		}
)

# Annotate for functions simply replace the body of the function
# with the annotated body.

setMethod("annotate", c("function"),  function(object, ...)  {
			body(object) <- annotate(body(object), ... )
			object
		}
)
