## RANDOM
## This file implements the combination of boxes and template parallelotopes without GMP

## Copyright 2008-2012 Gianluca Amato, Francesca Scozzari

## This file is part of RANDOM: R-based Analyzer for Numerical DOMains
##
## RANDOM is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## RANDOM is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty ofa
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with RANDOM.  If not, see <http://www.gnu.org/licenses/>.

combinedbox.domain <- function(M, aggregates=list()) {
	
	boxdom <- box.domain()
	rboxdom <- rotatedbox.domain(M, aggregates)
	
	transfer.info <- function(cbox) {
		size <- cbox$box$size
		rbox <- rboxdom$frombox(cbox$box,cbox$rbox$param)
		
		# per problemi di approssimazione potrebbe darsi che newbox diventi vuoto senza esserlo
		# questo controllo lo  impedisce 
		newbox <- rboxdom$intersection(rbox,cbox$rbox)
		if (! rboxdom$is.empty(newbox) ) rbox <- newbox else rbox <- cbox$rbox
		
		box <- rboxdom$tobox(rbox)
		# per problemi di approssimazione potrebbe darsi che newbox diventi vuoto senza esserlo
		# questo controllo lo  impedisce 
		newbox <- boxdom$intersection(box,cbox$box)
		if (! boxdom$is.empty(newbox)) box <- newbox else box <- cbox$box
		list (box=box, rbox=rbox)    
	}
	
	## Min / ArgMin
	
	argmin <- function (cbox, linear.form)  {
		dot1 <- boxdom$minime(cbox$box,linear.form)
		dot2 <- rboxdom$minimize(cbox$rbox,linear.form)
		if (dot1>=dot2)
			boxdom$argmin(cbox$box,linear.form)
		else
			rboxdom$argmin(cbox$rbox,linear.form)
	}
	
	argmax <- function (cbox, linear.form)  {
		dot1 <- boxdom$maximize(cbox$box,linear.form)
		dot2 <- rboxdom$maximize(cbox$rbox,linear.form)
		if (dot1<=dot2)
			boxdom$argmax(cbox$box,linear.form)
		else
			rboxdom$argmax(cbox$rbox,linear.form)
	}
	
	minimize <- function (cbox, linear.form) {
		dot1 <- boxdom$minimize(cbox$box,linear.form)
		dot2 <- rboxdom$minimize(cbox$rbox,linear.form)
		max(dot1,dot2)
	}
	
	maximize <- function (cbox, linear.form) {
		dot1 <- boxdom$maximize(cbox$box,linear.form)
		dot2 <- rboxdom$maximize(cbox$rbox,linear.form)
		min(dot1,dot2)
	}  	
	
	## generate new boxes from scratch
	full <- function(n=1) { 
		list(box=boxdom$full(n), rbox=rboxdom$full(n))
	}
	
	empty <- function(n=1) {
		list(box=boxdom$empty(n), rbox=rboxdom$empty(n))
	}
	
	zero <- function(n=1) {
		list(box=boxdom$zero(n), rbox=rboxdom$zero(n))
	}
	
	getsize <- function(cbox) {
		boxdom$getsize(cbox$box)
	}
	
	frombox <- function(box, newparam=NULL) {
		list(box=boxdom$frombox(box), rbox=rboxdom$frombox(box, newparam))
	}
	
	## Relations between boxes
	is.subseteq <- function(cbox1,cbox2) {
		boxdom$is.subseteq(cbox1$box, cbox2$box) && rboxdom$is.subseteq(cbox1$rbox, cbox2$rbox)
	}
	is.equal <- function(cbox1,cbox2) {
		boxdom$is.equal(cbox1$box, cbox2$box) && rboxdom$is.equal(cbox1$rbox, cbox2$rbox)
	}
	is.subset <- function(cbox1,cbox2) {
		boxdom$is.subset(cbox1$box, cbox2$box) || rboxdom$is.subset(cbox1$rbox, cbox2$rbox)
	}
	is.empty <- function(cbox) {
		boxdom$is.empty(cbox$box)
	}
	
	## Operations
	assign <- function(cbox, index, linear.form, newparam=NULL) {
		box <- boxdom$assign(cbox$box,index, linear.form)
		rbox <- rboxdom$assign(cbox$rbox,index, linear.form, newparam)
		transfer.info(list(box=box, rbox=rbox))
	}
	
	assign.unknown <- function(cbox, index) {
		list(
				box = boxdom$assign.unknown(cbox$box,index),
				rbox = rboxdom$assign.unknown(cbox$rbox,index)
		)
	}
	
	select <- function(cbox, linear.form) {
		transfer.info(list(
						box = boxdom$select(cbox$box,linear.form),
						rbox = rboxdom$select(cbox$rbox,linear.form))
		)
	}
	
	select.diff <- function(cbox, linear.form) {
		transfer.info(list(
						box = boxdom$select.diff(cbox$box,linear.form),
						rbox = rboxdom$select.diff(cbox$rbox,linear.form)))
	}
	
	union <- function(cbox1,cbox2) {
		list(
				box = boxdom$union(cbox1$box,cbox2$box),
				rbox = rboxdom$union(cbox1$rbox,cbox2$rbox)
		)
	}
	
	intersection <- function(cbox1,cbox2) {
		list(
				box = boxdom$intersection(cbox1$box,cbox2$box),
				rbox = rboxdom$intersection(cbox1$rbox,cbox2$rbox)
		)
	}
	
	widening <- function(cbox1,cbox2,delay=0) {
		list(
				box = boxdom$widening(cbox1$box,cbox2$box,delay),
				rbox = rboxdom$widening(cbox1$rbox,cbox2$rbox,delay)
		)
	}
	
	narrowing <- function(cbox1,cbox2) {
		list(
				box = boxdom$narrowing(cbox1$box,cbox2$box),
				rbox = rboxdom$narrowing(cbox1$rbox,cbox2$rbox)
		)
	}
	
	as.character <- function(cbox,vars) {
		paste(
				boxdom$as.character(cbox$box,vars),":",
				rboxdom$as.character(cbox$rbox,vars)
		)		
	}  
	
	setparam <- function(cbox, newparam) {
		list(box=cbox$box, rbox=rboxdom$setparam(cbox$rbox, newparam))
	}
	
	outputform <- function(box,lf,vars=paste("v",1:box$size,sep="")) {
		min.bound <- minimize(box,lf)
		max.bound <- maximize(box,lf)		
		boxdom$as.character(list(size=1, min=c(min.bound,1), max=c(max.bound,1)), linear.form.display(lf,vars))
	}
	
	list (full = full, empty = empty, zero=zero, getsize=getsize, frombox=frombox,
			is.subseteq = is.subseteq, is.subset = is.subset, is.equal = is.equal, is.empty = is.empty,
			argmin = argmin, argmax= argmax,
			minimize=minimize, maximize=maximize,
			assign = assign,  assign.unknown = assign.unknown,
			select = select, select.diff = select.diff, union = union,
			intersection = intersection, widening = widening,
			narrowing=narrowing, as.character=as.character, setparam=setparam, outputform=outputform )
}
