## RANDOM
## This file implements template parallelotopes without GMP

## Copyright 2008-2011 Gianluca Amato, Francesca Scozzari

## This file is part of RANDOM: R-based Analyzer for Numerical DOMains
##
## RANDOM is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## RANDOM is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty ofa
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with RANDOM.  If not, see <http://www.gnu.org/licenses/>.

rotatedbox.domain <- function(M, aggregates=list()) {
	
	normal.and.inverse <- function(M) {
		if (is.null(M)) return(NULL)
		if (attr(M,"inverse")) {
			invA <-  rbind(cbind(M, rep(0, nrow(M))),c(rep(0,nrow(M)),1))
			A <- solve(invA)
		} else {
			A <- rbind(cbind(M, rep(0, nrow(M))),c(rep(0,nrow(M)),1))
			invA <- solve(A)
		}
		list(normal=A, inverse=invA)
	}	
	
	default <- if (length(aggregates)==0) normal.and.inverse(M) else list(normal=diag(ncol(M)+1),inverse=diag(ncol(M)+1))
	aggregates <- lapply(aggregates, normal.and.inverse)
	boxdom <- box.domain()
	
	select.matrices <- function(newparam=NULL, rbox) {		
		if (is.null(newparam)) 
			newparam <- rbox$param
		if (is.null(newparam)) 
			default 
		else if (newparam <=length(aggregates) && !is.null(aggregates[[newparam]])) 
			aggregates[[newparam]]
		else 
			default
	}
	
	frombox <- function(box, newparam=NULL) {
		size <- boxdom$getsize(box)
		rbox <- boxdom$full(size)
		rbox$param <- newparam		
		invA <- select.matrices(newparam,rbox)$inverse
		for(i in 1:size) {
			rbox$min[i] <- boxdom$minimize(box,invA[i,])
			rbox$max[i] <- boxdom$maximize(box,invA[i,])
		}
		rbox
	}
	
	tobox <- function(rbox) {
		A <- select.matrices(rbox$param, rbox)$normal
		size <- getsize(rbox)
		box <- boxdom$full(size)
		for (i in 1:size) {
			box$min[i] <- boxdom$minimize(rbox,A[i,])
			box$max[i] <- boxdom$maximize(rbox,A[i,])      
		}
		box
	}
	
	getsize <- function(rbox) {
		rbox$size
	}
	
	## Min / ArgMin
	## Non è possibile da implementare in maniera corretta sui FP
	
	argmin <- function (rbox, linear.form)  {
		A <- select.matrices(rbox$param,rbox)$normal
		linear.form <- t(A) %*% linear.form    
		ifelse(linear.form >= 0, rbox$min, rbox$max)  
	}
	
	argmax <- function (rbox, linear.form)  {
		A <- select.matrices(rbox$param,rbox)$normal
		linear.form <- t(A) %*% linear.form    
		ifelse(linear.form >=0, rbox$max, rbox$min)  
	}
	
	minimize <- function (rbox, linear.form) {
		A <- select.matrices(rbox$param,rbox)$normal
		linear.form <- t(A) %*% linear.form
		min.bounds <- ifelse(linear.form >= 0, rbox$min, rbox$max)  
		dot.product(linear.form, min.bounds)
	}
	
	maximize <- function (rbox, linear.form) {		
		A <- select.matrices(rbox$param,rbox)$normal
		linear.form <- t(A) %*% linear.form    
		max.bounds <- ifelse(linear.form >=0, rbox$max, rbox$min)  
		dot.product(linear.form, max.bounds)
	}  
	
	## Operations
	assign <- function(rbox, index, linear.form, newparam=NULL) {	
		if (boxdom$is.empty(rbox)) return(rbox)		
		linear.form[index] <- linear.form[index]-1
		if (is.null(newparam)) {
			A <- select.matrices(rbox$param,rbox)$normal
			invA <- select.matrices(rbox$param,rbox)$inverse
			H <- diag(nrow(A)) + invA[,index] %*% (t(linear.form) %*% A)
			newparam <- rbox$param
		} else {
			A <- select.matrices(rbox$param,rbox)$normal
			invA <- select.matrices(newparam,rbox)$inverse
			ei <- replace(rep(0,nrow(A)),c(index, nrow(A)),c(1,1))
			H <- invA %*% (diag(nrow(A)) +  ei %*% t(linear.form)) %*% A
		}
		size <- rbox$size
		newbox <- boxdom$full(size)
		newbox$param <- newparam
		for (i in 1:size) {
			min.bound <- boxdom$argmin(rbox,H[i,])
			max.bound <- boxdom$argmax(rbox,H[i,])
			newbox$min[i] <- dot.product(H[i,],min.bound)
			newbox$max[i] <- dot.product(H[i,],max.bound)
		}
		newbox
	}
	
	assign.unknown <- function(rbox, index) {
		invA <- select.matrices(rbox$param,rbox)$inverse				
		if (boxdom$is.empty(rbox)) return(rbox)
		rbox$min = ifelse(invA[,index]==0, rbox$min, -Inf)
		rbox$max = ifelse(invA[,index]==0, rbox$max, +Inf)		
		rbox
	}
	
	select = function(rbox, linear.form) {
		A <- select.matrices(rbox$param,rbox)$normal
		linear.form <- t(A) %*% linear.form
		rboxnew <- boxdom$select(rbox,linear.form)
		rboxnew$param <- rbox$param
		rboxnew
	}
	
	select.diff <- function(rbox, linear.form) {
		A <- select.matrices(rbox$param,rbox)$normal
		linear.form <- t(A) %*% linear.form
		rboxnew <- boxdom$select.diff(rbox,linear.form)
		rboxnew$param <- rbox$param
		rboxnew
	}
	
	as.character <- function(rbox,vars=paste("v",1:rbox$size,sep="")) {
		invA <- select.matrices(rbox$param,rbox)$inverse		
		vars.original <- apply(invA,1,function (lf) linear.form.display(lf,vars,digits=3))
		vars.original <- vars.original[-length(vars.original)]
		paste(boxdom$as.character(rbox, vars.original)," ( ",rbox$param," ) ")
	}
	
	outputform <- function(rbox,lf,vars=paste("v",1:box$size,sep="")) {
		min.bound <- minimize(rbox,lf)
		max.bound <- maximize(rbox,lf)		
		boxdom$as.character(list(size=1, min=c(min.bound,1), max=c(max.bound,1)), linear.form.display(lf,vars))
	}
	
	setparam <- function(rbox, newparam) {
		if (identical(newparam,rbox$param) || is.null(newparam)) return(rbox)
		size <- boxdom$getsize(rbox)
		newrbox <- boxdom$full(size)
		newrbox$param <- newparam		
		invNEW <- select.matrices(newparam,rbox)$inverse
		normalOLD <- select.matrices(rbox$param,rbox)$normal		
		irot <- invNEW %*% normalOLD
		for (i in 1:newrbox$size) {
			newrbox$min[i] <- boxdom$minimize(rbox,irot[i,])
			newrbox$max[i] <- boxdom$maximize(rbox,irot[i,])
		}		
		newrbox
	}
	
	intersection <- function(rbox1,rbox2) {
		if (boxdom$is.empty(rbox1)) return(rbox1)
		if (boxdom$is.empty(rbox2)) return(rbox2)
		rbox1new <- setparam(rbox1,rbox2$param)
		rbox <- boxdom$intersection(rbox1new,rbox2)
		rbox$param <- rbox2$param
		rbox
	}
	
	union <- function(rbox1,rbox2) {
		if (boxdom$is.empty(rbox1)) return(rbox2)
		if (boxdom$is.empty(rbox2)) return(rbox1)
		rbox1new <- setparam(rbox1,rbox2$param)		
		rbox <- boxdom$union(rbox1new,rbox2)
		rbox$param <- rbox2$param
		rbox
	}
	
	widening <- function(rbox1, rbox2, delay=0) {
		rbox1new <- setparam(rbox1,rbox2$param)
		rbox <- boxdom$widening(rbox1new,rbox2,delay)
		rbox$param <- rbox2$param
		rbox
	}
	
	narrowing <- function(rbox1, rbox2) {
		rbox1new <- setparam(rbox1,rbox2$param)
		rbox <- boxdom$narrowing(rbox1new,rbox2)
		rbox$param <- rbox2$param
		rbox
	}
	
	list (  full = boxdom$full, empty = boxdom$empty, zero=boxdom$zero, frombox=frombox,
			getsize=boxdom$getsize, tobox=tobox, getsize=getsize,
			is.subseteq = boxdom$is.subseteq, is.subset = boxdom$is.subset,
			is.equal = boxdom$is.equal, is.empty =  boxdom$is.empty,
			argmin = argmin, argmax = argmax,
			minimize = minimize, maximize = maximize,
			assign = assign, assign.unknown = assign.unknown,        
			select = select, select.diff = select.diff,
			union = union, intersection = intersection,
			widening = widening, narrowing=narrowing,
			as.character=as.character, setparam=setparam, 
			outputform=outputform )
}
