## RANDOM
## This is the editor for matrices.

## Copyright 2010, 2011 Gianluca Amato, Francesca Scozzari

## This file is part of RANDOM: R-based Analyzer for Numerical DOMains
##
## RANDOM is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## RANDOM is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with RANDOM.  If not, see <http://www.gnu.org/licenses/>.

editdialog.new <- function(state,builder) {
	dialog <- builder$getObject("editdialog")
	
	manual.matrix <- NULL

	initializeMatrix <- function(vars) {
		identity <- statisticalTraceAnalyzer$identity(call=NULL, vars=vars)$matrix
		manual.matrix <<- identity
	}
					
	getMatrix <- function () {
		manual.matrix
	}
			
	setMatrix <- function(m) {
		if (any(is.na(m))) {
			error.dialog("The matrix contains invalid values","matrix validation")
			return(FALSE)
		}
		result <- tryCatch (
				{
					inv <- solve(m)
					TRUE
				},				
				error = function(cond) {
					error.dialog("The transformation matrix is singular","matrix validation")
					return(FALSE)
				}
		)
		if (! result) return(FALSE)	
	    manual.matrix <<- m
		return(TRUE)
	}
		
	run <- function () {		
		# callback function
				
		loadfromcurrent_cb <- function(button,data) {
			entries.set(state$getSTA()$matrix)
		}
		
		inverse_cb <- function(widget,data) {
			m <- entries.get()
			m <- t(m)
			entries.set(m)
		}
		
		# utility functions
		
		entries.set <- function(m) {			
			gSignalHandlerBlock(checkbutton_inverse,handler_inverse)
			checkbutton_inverse$setActive(attr(m,"inverse"))
			gSignalHandlerUnblock(checkbutton_inverse,handler_inverse)
			
			for (i in 1:nr)
				for (j in 1:nc) {
					entries[[(i-1)*nr+j]]$setText(m[i,j])
					entries[[(i-1)*nr+j]]$modifyText("normal", NULL)					
				}
			for (i in 1:nr)
				rowlabels[[i]]$setText(rownames(m)[i])		
			for (j in 1:nc)
				collabels[[j]]$setText(colnames(m)[j])				
		}
		
		entries.get <- function() {	
			m <- matrix("",nrow=nr,ncol=nc)
			rownames(m) <- character(nr)
			colnames(m) <- character(nc)
			for (i in 1:nr)
				for (j in 1:nc) 
					m[i,j] <- entries[[(i-1)*nr+j]]$getText()
			for (i in 1:nr)
				rownames(m)[i] <- rowlabels[[i]]$getText()
			for (j in 1:nr)
				colnames(m)[j] <- collabels[[j]]$getText()
			attr(m,"inverse") <- checkbutton_inverse$getActive()
			return(m)
		}
	
		m <- getMatrix()
		if (is.null(m)) return(FALSE)
		
		# connect signal handlers		
		button_current <- builder$getObject("editdialog_loadfromcurrent")
		handler_current <- gSignalConnect(button_current,"clicked",loadfromcurrent_cb)		
		checkbutton_inverse <- builder$getObject("checkbutton_inverse")
		handler_inverse <- gSignalConnect(checkbutton_inverse,"toggled",inverse_cb)
		
				
		nr <- nrow(m)
		nc <- ncol(m)	
		table <- gtkTable(nr+1,nc+1)
		entries <- list()
		rowlabels <- list()
		collabels <- list()
		for (i in 1:nr)
			for (j in 1:nc) {
				entry <- gtkEntryNew()
				entry$setSizeRequest(0,-1)
				table$attach(entry,j,j+1,i,i+1)
				entries[[(i-1)*nr+j]] <- entry
			}	
		for (i in 1:nr) {
			rowlabels[[i]] <- gtkLabel()
			table$attach(rowlabels[[i]],0,1,i,i+1,"","",xpadding=5)
		}
		for (i in 1:nc) {
			collabels[[i]] <- gtkLabel()			
			table$attach(collabels[[i]],i,i+1,0,1,ypadding=5)
		}
		entries.set(m)			
		dialog$vbox$add(table)	
		
		# entries.wrong is a boolean matrix.. TRUE means the corresponding entry

entries.wrong <- matrix(FALSE, nrow=nr, ncol=nc)
		
		# m.invalid is TRUE if the m entered by the user is invalid for
		# any reason
		m.invalid <- TRUE
		
		while (m.invalid) {
			# colors with red the wrong entries
			for (i in 1:nr)
				for (j in 1:nc) 
					if (entries.wrong[i,j]) 
						entries[[(i-1)*nr+j]]$modifyText("normal", "red")
					else
						entries[[(i-1)*nr+j]]$modifyText("normal", NULL)
			
			response <- dialog$run()	
			if (response == GtkResponseType["ok"]) {
				m <- entries.get()
				for (i in 1:nr)
					for (j in 1:nc) {
						entry <- entries[[(i-1)*nr+j]]
						tryCatch (
								{
									m[i,j] <- as.double(entry$getText())
									entries.wrong[i,j] <- FALSE
								},
								warning = function(cond) entries.wrong[i,j] <<- TRUE
						)
					}		
				if (any(entries.wrong)) 
					error.dialog("The matrix contains invalid values","matrix validation")
				else {
					storage.mode(m)="double"
					m.invalid <- ! setMatrix(m)	
				}
			} else 
				break;		
		}
		
		gSignalHandlerDisconnect(button_current,handler_current)		
		gSignalHandlerDisconnect(checkbutton_inverse,handler_inverse)
		
		table$destroy()
		dialog$hide()
		return(response == GtkResponseType["ok"])
	}	
	
	list(run = run, getMatrix = getMatrix, initializeMatrix = initializeMatrix)	
}
