## S3 class bigqi testing
## Test class for bigqi

## Copyright 2008-2010 Gianluca Amato, Francesca Scozzari

## This file is part of RANDOM: R-based Analyzer for Numerical DOMains
##
## RANDOM is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## RANDOM is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with RANDOM.  If not, see <http://www.gnu.org/licenses/>.

source("bigqi.r")

test.bigqi <- function() {

  exact.compare <- function(v1,v2) {
     if (class(v2)=="bigqi") 
       check <- bigqi.check(v2)
     else c
       check <- TRUE
     ifelse(is.na(v1),is.na(v2),v1==v2) & check
  }

  bigq <- as.bigq(   c( 0,0,0, 0, 2,2,2, 2,NA,NA,NA,NA))
  inf  <- as.integer(c(NA,0,1,-1,NA,0,1,-1,NA, 0, 1,-1))
  check.res <-       c( F,T,T, T, F,T,F, F, T, F, F, F)
  qi <- bigqi.make(bigq,inf)
  cat("Check consistency functions\n")
  print(exact.compare(check.res,bigqi.check(qi)))


  x <- as.bigqi(c())
  y <- as.bigqi(c(2,56,124,3))  
  cat("\nZero-length operations\n")
  print(c(length(x+y)==0,length(x-y)==0,length(x*y)==0,length(x/y)==0))

  x <- as.bigqi(c(2,5,8,4))
  cat("\nNon zero-length operations\n")
  print(c(length(x+y)==4,length(x-y)==4,length(x*y)==4,length(x/y)==4))

  cat("\nEmpty Repetitions:\n")
  print(rep(x,0)==bigqi(0))

  cat("\nNon Empty Repetitions 1:\n")
  print(exact.compare(rep(x,1),as.bigqi(c(2,5,8,4))))
   
  cat("\nNon Empty Repetitions 2:\n")
  print(exact.compare(rep(x,2),as.bigqi(c(2,5,8,4,2,5,8,4))))
  
 
  x         <- as.bigqi(c(0 , 0,  0,   0,  0,  2,2,  2,   2,  2,Inf,Inf,Inf, Inf,Inf,-Inf,-Inf,-Inf,-Inf,-Inf,NA,NA, NA,  NA,NA))
  y         <- as.bigqi(c(0 , 2, Inf,-Inf,NA,  0,2, Inf,-Inf,NA,  0,  2,Inf,-Inf, NA,   0,   2, Inf,-Inf,  NA, 0, 2,Inf,-Inf,NA))
  sum.res   <- as.bigqi(c(0 , 2, Inf,-Inf,NA,  2,4, Inf,-Inf,NA,Inf,Inf,Inf,  NA, NA,-Inf,-Inf,  NA,-Inf,  NA,NA,NA, NA,  NA,NA))
  minus.res <- as.bigqi(c(0 ,-2,-Inf, Inf,NA,  2,0,-Inf, Inf,NA,Inf,Inf, NA, Inf, NA,-Inf,-Inf,-Inf,  NA,  NA,NA,NA, NA,  NA,NA))
  mul.res   <- as.bigqi(c(0 , 0,  NA,  NA,NA,  0,4, Inf,-Inf,NA, NA,Inf,Inf,-Inf, NA,  NA,-Inf,-Inf, Inf,  NA,NA,NA, NA,  NA,NA))
  div.res   <- as.bigqi(c(NA, 0,   0,   0,NA,Inf,1,   0,   0,NA,Inf,Inf, NA,  NA, NA,-Inf,-Inf,  NA,  NA,  NA,NA,NA, NA,  NA,NA))
  less.res  <-          c(F,  T,   T,   F,NA,  F,F,   T,   F,NA,  F,  F,  F,   F, NA,   T,   T,   T,   F,  NA,NA,NA, NA,  NA,NA)
  leq.res   <-          c(T,  T,   T,   F,NA,  F,T,   T,   F,NA,  F,  F,  T,   F, NA,   T,   T,   T,   T,  NA,NA,NA, NA,  NA,NA)

  cat("\nSum:\n")
  print(exact.compare(sum.res,x+y))
  cat("\nDifference:\n")
  print(exact.compare(minus.res,x-y))
  cat("\nOther Operations: 3-NA, 3*NA\n")
  print(exact.compare(as.bigqi(3)-NA,NA))
  print(exact.compare(as.bigqi(3)*NA,NA))
  cat("\nMultiplication:\n")
  print(exact.compare(mul.res,x*y))
  cat("\nDivision:\n")
  print(exact.compare(div.res,x/y))
  cat("\nLess then:\n")
  print(exact.compare(less.res,x<y))
  cat("\nLess or equal then:\n")
  print(exact.compare(leq.res,x<=y))
  
}




