## Copyright 2008-2012 Gianluca Amato, Francesca Scozzari

## This file is part of RANDOM: R-based Analyzer for Numerical DOMains
##
## RANDOM is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## RANDOM is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty ofa
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License

plottingview <- proto(
		init = function(., model, builder) {			
			var_changed_cb <- function(widget, user_data) {
				if (! model$getRecomputeTrace()) .$redraw()
				return(TRUE)
			} 
			.$model <- model
			.$colors <- c("black","red")
			.$window <- builder$getObject("plottingWindow")
			.$varsStoreSelected <- builder$getObject("varsStoreSelected")			
			gSignalConnect(.$window,"delete-event", function(widget, event, user_data) { 					
						.$model$setGraphicsFlag(FALSE)
						return(TRUE)
					})
			.$xvar <- builder$getObject("plottingWindowXVar")
			.$xhandler <- gSignalConnect(.$xvar,"changed", var_changed_cb)
			.$yvar <- builder$getObject("plottingWindowYVar")
			.$yhandler <- gSignalConnect(.$yvar,"changed", var_changed_cb)
			da <- builder$getObject("plottingDrawingArea")							
			asCairoDevice(da)			
		},
		draw = function(.) {
			vars <- .$model$getVariables()
			gSignalHandlerBlock(.$xvar,.$xhandler)
			gSignalHandlerBlock(.$yvar,.$yhandler)
			.$varsStoreSelected$clear()
			for (v in vars) {
				i <- .$varsStoreSelected$append()
				.$varsStoreSelected$setValue(i$iter, 0, v)
			}
			if (length(vars)==0) {
				# It should never happend... just in case
				.$xvar$setActive(-1)
				.$yvar$setActive(-1)
			} else if (length(vars)==1) {
				.$xvar$setActive(0)
				.$yvar$setActive(0)
			} else {
				.$xvar$setActive(0)
				.$yvar$setActive(1)
			}				
			gSignalHandlerUnblock(.$xvar,.$xhandler)
			gSignalHandlerUnblock(.$yvar,.$yhandler)
			.$redraw()
		},
		hide = function(.) {
			.$window$hide()		
		},
		show = function(.) {
			.$window$show()
		},
		destroy = function(.) {
			.$window$destroy()
		},
		redraw = function(.) {	
			data <- .$model$getTrace()$getData()

			if (.$xvar$getActive()==-1 || .$yvar$getActive()==-1) {
				# It should never happen... just in case
				plot(logical(0),xlim = c(0,1), ylim=c(0,1), xlab="",ylab="",axes=FALSE)
				return()
			}
			col1 <- .$xvar$getActive() +2
			col2 <- .$yvar$getActive() +2
			
			minLimit <- min(0,min(data[,c(col1,col2)],na.rm=TRUE))
			maxLimit <- max(0,max(data[,c(col1,col2)],na.rm=TRUE))
			
			diff <- abs(maxLimit-minLimit)
			limit <- c(minLimit-0.3*diff,maxLimit+0.3*diff)
			plot(logical(0),xlim=limit,ylim=limit,xlab="",ylab="",axes=FALSE)
			
			minLimit2 <- minLimit-0.2*diff
			maxLimit2 <- maxLimit+0.2*diff
			
			minLimit1 <- minLimit-0.1*diff
			maxLimit1 <- maxLimit+0.1*diff
			
			arrows(0,minLimit1,0,maxLimit1,code=2,length=.1)
			text(maxLimit2,0,colnames(data)[col1])
			arrows(minLimit1,0,maxLimit1,0,code=2,length=.1)
			text(0,maxLimit2,colnames(data)[col2])
			
			programPoints <- unique(data[,1])
			
			for (pp in programPoints) {
				selectedPoints=which(data[,1]==pp)
				points(data[selectedPoints,c(col1,col2)],col=pp+1)		

			}
			m <- .$model$getSTA()$matrix

			if (is.null(attr(m,"inverse")) || (attr(m,"inverse")==FALSE)) 
				m <- t(m)												
			for(i in 1:ncol(m)) {
				component <- m[i,]
				a=component[col1-1]
				b=component[col2-1]
				if(b==0)
					abline(v=0,col=i,lty=3)
				else
					abline(0,-a/b,col=i,lty=3)
			}
			legend("bottomleft", row.names(m),lty=3,col=c(1:ncol(m)),cex=0.8)
		}
)
