## RANDOM
## This is the entry-point. It loads all the other files which make up
## the static analyzer RANDOM.

## Copyright 2008-2012 Gianluca Amato, Francesca Scozzari

## This file is part of RANDOM: R-based Analyzer for Numerical DOMains
##
## RANDOM is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## RANDOM is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with RANDOM.  If not, see <http://www.gnu.org/licenses/>.

suppressPackageStartupMessages({		
	library(gmp)
	library(RGtk2)
	library(cairoDevice)
	library(ieeeround)
	library(fastICA)
	library(proto)
})

GMPOK <- "0.5.1"   # Least tested version of GMP

if (packageVersion("gmp") < GMPOK) 
	message("Unsupported version of gmp library. Please update to ",GMPOK," or later.")

source("libraries/gcd.R")
source("libraries/bigqi.R")
source("libraries/solve.trick.R")
source("domains/domains.R", chdir=TRUE)
source("simplify.R")
source("annotate.R")
source("tracer.R")
source("trace-analyzers.R")
source("analyzer.R")
source("editdialog.R")
source("varsdialog.R")
source("plotting.R")

## callback functions

notebook_switch_page_cb = function (widget, page, page_num, user_data) {	
	if (page_num==2) {		
		m <- state$getSTA()$matrix		
		if (is.null(attr(m,"inverse")) || (attr(m,"inverse")==FALSE)) 
			m=t(m)
		attr(m,"inverse") <- NULL
		txt1 <- paste(capture.output(print(m)),collapse="\n")		
		a <- state$getSTA()$aggregates		
		if (length(a)!=0)
			txt2 <- paste(capture.output(print(state$getSTA()$aggregates)),collapse="\n")
		else
			txt2 <- ""
		matrixBuffer$setText(paste(txt1,txt2,sep="\n"))
	}			
	else if (page_num==3)
		script.trace()	
}

on_activeGraphics_toggled = function(widget,user.data) {
	if (state$getGraphicsFlag())  
		plottingview$show()	
	else 
		plottingview$hide()			
}


scriptBuffer_modified_changed_cb <- function(text_buffer,user_data) {
   	mainWindow.changeTitle()	
}

scriptBuffer_changed_cb <- function(text_buffer,user_data) {
	state$setRecomputeParse(TRUE)
}

button_edit_clicked_cb <- function(widget,user.data) {
	if (state$getInitializeManualMatrix()) {
		if (! script.parse()) return(NULL)
		editdialog$initializeMatrix(state$getVariables())						
		state$setInitializeManualMatrix(FALSE)	
	}				
	if (editdialog$run()) state$setRecomputeMatrix(TRUE, force="manual")			
}

button_variables_clicked_cb <- function(widget,user.data) {
	
	if (varsdialog$run() == GtkResponseType["ok"]) state$setRestrictVarsFlag(TRUE)	
}

mainWindow.delete <- function(window, userdata)
{
	if (! unsaved.dialog("Close") ) return(TRUE)
	state$getMainWindow()$destroy()
	plottingview$destroy()
	gtkMainQuit()
}

mainWindow.changeTitle <- function() {
	title <- paste("RANDOM --",state$getScriptName())
	if (state$getModifiedAfterSave()) 
		title <- paste("*",title,sep="")
	state$getMainWindow()$setTitle(title)
}

unsaved.dialog <- function(operation)
{
	if (! state$getModifiedAfterSave())
		return(TRUE)
	firstlabel <- paste(operation,"without Saving")
	if (state$isNamed())
		lastlabel <- "Save"
	else
		lastlabel <- "Save As"
	dialog <- gtkMessageDialog(state$getMainWindow(),"destroy-with-parent","warning",
			"none","Save changes to script \"",state$getScriptName(),"\" ?",sep="",
			show=FALSE)
	dialog$formatSecondaryText("If you don't save, last changes will be permanently lost.")
	dialog$addButtons(firstlabel,GtkResponseType["accept"],
			"Cancel",GtkResponseType["cancel"],lastlabel,GtkResponseType["reject"])
	response <- dialog$run()
	dialog$destroy()	
	if (response == GtkResponseType["cancel"])
		return(FALSE)
	else if (response == GtkResponseType["reject"])
		return(script.save(widget,userdata)) 	
	else return(TRUE)
}

error.dialog <- function(message,subsystem) {
	dialog <- gtkMessageDialog(state$getMainWindow(),"destroy-with-parent","error",
			"ok",message)
	if (missing(subsystem))
		dialog$setTitle("Error")
	else
		dialog$setTitle(paste("Error during",subsystem))
	dialog$run()
	dialog$destroy()
}

script.new <- function(widget, userdata)
{
	if (! unsaved.dialog("New")) return(FALSE)
	state$setScript("")
	state$setScriptName()
	return(TRUE) 	  			
}

script.save <- function(widget, userdata, saveas=FALSE)
{
	if (! state$isNamed() || saveas) {		
		fc <- gtkFileChooserDialog("Save Script",state$getMainWindow(),"save",
				GTK_STOCK_CANCEL, GtkResponseType["cancel"],
				GTK_STOCK_SAVE, GtkResponseType["accept"])
		response <- fc$run()
		filename <- fc$getFilename()		
		fc$destroy()		
	} else {
		filename <- state$getScriptName()
		response <- GtkResponseType["accept"]
	}	
	if (response == GtkResponseType["accept"]) {
		writeLines(state$getScript(),filename)
		state$setScriptName(filename)
		state$notifySaved()
		return(TRUE)
	} else
		return(FALSE)
}

script.saveas <- function(widget, userdata) { 
	script.save(widget,userdata,saveas=TRUE)
}

script.open <- function(widget, userdata)
{
	if (!unsaved.dialog("Open")) return(FALSE) 		
	fc <- gtkFileChooserDialog("Open Script",state$getMainWindow(),"open",
			GTK_STOCK_CANCEL, GtkResponseType["cancel"],
			GTK_STOCK_OPEN, GtkResponseType["accept"])	
	response <- fc$run()
	filename <- fc$getFilename()
	fc$destroy()
	if (response == GtkResponseType["accept"]) 
		state$loadScript(filename)	
	return(TRUE)
}

script.cut <- function(widget, userdata) {
	state$cut()
}

script.copy <- function(widget, userdata) {
	state$copy()
}

script.paste <- function(widget, userdata) {
	state$paste()
}

script.delete <- function(widget, userdata) {
	state$delete()	
}

script.parse <- function()
{	
	if (! state$getRecomputeParse() ) return(TRUE)
	state$setStatusMessage("Parsing in progress...", TRUE)
	state$setClosure(NULL)
	state$setCases(list(list())) 
	status <- tryCatch(
		{								
			result <- eval(parse(text=state$getScript()),new.env())
			if (mode(result) == "function") { 
				state$setClosure(result)				
				TRUE
			} else if (mode(result) == "list" && length(result)<=2  
					&& mode(result[[1]])=="function") {
				state$setClosure(result[[1]])
				if (length(result) == 2) 
					state$setCases(as.list(result[[2]]))				
				TRUE
			} else {
				error.dialog("Script should evaluate to a closure or a list whose first element is a closure","parsing")
				FALSE
			}				
		},
		error = function (cond) {			
			error.dialog("Invalid parsing of script","parsing")
			return(FALSE)
		}
	)		
	state$setStatusMessage("", TRUE)
	return(status)
}

 script.trace <- function(onlyforvars=FALSE,graphicFlag=FALSE) {	
	if (! state$getRecomputeTrace() ) return(TRUE)
	if (! script.parse()) return(FALSE)
	closure <- state$getClosure()
	cases <- state$getCases()
	vars <- state$getVariables()
	if (length(formals(closure))!=0 && is.null(cases)) {
		error.dialog("You need to specify the value of formal variables","tracing")
		return(FALSE)
	}			
	if (length(vars)==0) {
		error.dialog("Please, select at least a variable for the analysis")
		return(FALSE)
	}
	tag <- state$getTagChoice()	
	state$setStatusMessage("Trace collection in progress...", TRUE)
	trace <- tryCatch (
		programTracer$fromFunction(closure,cases,vars=state$getVariables(), tag=tag),
		error = function(cond) {
			error.dialog(conditionMessage(cond),"tracing")
			NULL
		}
	)
	state$setStatusMessage("", TRUE)
	if (is.null(trace)) return(FALSE)
	state$setTrace(trace,graphicFlag)
	return(TRUE)
}

script.analyze <- function(widget,user.data) {
	if (! (state$selectedDomain("parallelotopes") || state$selectedDomain("combined")
				||  state$selectedDomain("intervals") || state$selectedDomain("generalptopes"))) {
		error.dialog("Please, select at least an abstract domain")
		return(FALSE)
	}
	if (! script.parse()) return(FALSE)
		
	startzero <- if (state$getLocalZero()) "local"  else "no"
	debug <- state$getDebugFlag()
	
	tryCatch( { 
	
	if (state$selectedDomain("parallelotopes") || state$selectedDomain("combined") || state$getGraphicsFlag())  {
		sta <- state$getSTA()
		matrix <- sta$matrix
		vars <- rownames(matrix)
		inverse <- attr(matrix,"inverse")		
		if (inverse) 
			vars <- colnames(matrix)
		else 
			vars <- rownames(matrix)		
	} else {
		vars <- state$getVariables()
		sta <- NULL
	}
	
	if (length(vars)==0) {
		error.dialog("Please, select at least a variable for the analysis")
		return(FALSE)
	}
	
	analyzed.call <- state$getClosure()
	if (state$getPerPP() && !is.null(sta)) body(analyzed.call) <-  sta$call
	
	linearforms <- state$getLinearForms()
	results <- list()
	if (state$selectedDomain("intervals")) {
		if (state$getGMP())
			domain <- box.domain.gmp()
		else
			domain <- box.domain()
		state$setStatusMessage("Analysis with intervals in progress...", TRUE)
		results[[1]] <- analyze.function(analyzed.call,  domain, vars=vars, startzero=startzero, debug.analysis=debug, displayed.forms=linearforms)
	}
	if (state$selectedDomain("parallelotopes")) {
		if (state$getGMP())
			domain <- rotatedbox.domain.gmp(matrix,sta$aggregates)
		else
			domain <- rotatedbox.domain(matrix,sta$aggregates)
		state$setStatusMessage("Analysis with parallelotopes in progress...", TRUE)
		results[[2]] <- analyze.function(analyzed.call, domain, vars=vars, startzero=startzero, debug.analysis=debug, displayed.forms=c(vars,linearforms))
	}
	if (state$selectedDomain("combined")) {		
		if (state$getGMP())
			domain <- combinedbox.domain.gmp(matrix,sta$aggregates)
		else
			domain <- combinedbox.domain(matrix,sta$aggregates)
		state$setStatusMessage("Analysis with combined domain in progress...", TRUE)
		results[[3]] <- analyze.function(analyzed.call, domain, vars=vars, startzero=startzero, debug.analysis=debug, displayed.forms=linearforms)
	}    	
	if (state$selectedDomain("generalptopes")) {		
		if (state$getGMP())
			domain <- generalptopes.domain.gmp()    # generalptopes.domain.gmp is not yet available
		else
			domain <- generalptopes.domain()
		state$setStatusMessage("Analysis with general parallelotopes domain in progress...", TRUE)
		results[[4]] <- analyze.function(analyzed.call, domain,vars=vars, startzero=startzero, debug.analysis=debug, displayed.forms=linearforms)
	}   
	textResults <- lapply(results, function(result) if (is.null(result)) NULL else format(result$out))	
	state$setResult(textResults)
	state$setStatusMessage("", TRUE)
	}, 
	simpleCondition=function(e) error.dialog(conditionMessage(e)),
	error = function(e) error.dialog(paste("An error occured which should have never happened.",
						"You hit a bug of either RANDOM or R.\n\nERROR: ",conditionMessage(e))),
	finally=state$setStatusMessage("", TRUE)
	)# end tryCatch
	
}

random.about <- function(widget,userdata) {	
	if (is.null(state$aboutDialog))
		state$aboutDialog <- builder$getObject("aboutdialog")
	dialog <- state$aboutDialog	
	dialog$run()
	dialog$hide()
}

trace.modified.state <- function (widget,userdata) {
	state$setRecomputeTrace(TRUE)
}

matrix.modified.state <- function (widger,userdata) {	
	state$setRecomputeMatrix(TRUE)	
}

state.new <- function () {
	mainWindow <- builder$getObject("mainWindow")
	notebook <- builder$getObject("notebook")	
	scriptBuffer <- builder$getObject("scriptBuffer")
	resultBuffer <- builder$getObject("resultBuffer")
	traceBuffer <-  builder$getObject("traceBuffer")
	statusBar <- builder$getObject("statusbar")
	gmpCheckButton <- builder$getObject("gmpcheckbutton")
	checkbuttonLocalzero <- builder$getObject("localzerocheckbutton")	
	checkbuttonOrthogonal <- builder$getObject("orthogonalcheckbutton")
	checkbuttonReverse <- builder$getObject("reversecheckbutton")
	tagComboBox <- builder$getObject("tagComboBox")
	thresholdAdjustment <- builder$getObject("threshold")	
	
	comboboxForceInteger <- builder$getObject("combobox_forceinteger")
	checkbuttonTranspose <- builder$getObject("checkbutton_transpose")
	checkbuttonIntervals <- builder$getObject("checkbutton_intervals")
	checkbuttonParallelotopes <- builder$getObject("checkbutton_parallelotopes")	
	checkbuttonCombined <- builder$getObject("checkbutton_combined")
	checkbuttonGeneralPTopes <- builder$getObject("checkbutton_generalptopes")
	checkbuttonRestrictVars <- builder$getObject("checkbutton_restrictvars")
	checkbuttonDebug <- builder$getObject("checkbutton_debug")
	checkbuttonPerPP <- builder$getObject("checkbutton_perpp")
	entryLinearForms <- builder$getObject("entry_linearforms")
	checkbuttonGraphics <- builder$getObject("activeGraphics")
	
	radiobuttonPCA <- builder$getObject("radiobutton_pca")
	radiobuttonICA <- builder$getObject("radiobutton_ica")	
	radiobuttonManual <- builder$getObject("radiobutton_manual")
	
	radiobuttonSimplifyOSCA <- builder$getObject("radiobutton_simplify_osca")
	radiobuttonGCDSimplification <- builder$getObject("radiobutton_simplify_gcd")
	
	checkbuttonIntervals$getChild()$modifyFg("active","blue")
	checkbuttonParallelotopes$getChild()$modifyFg("active","red")
	checkbuttonCombined$getChild()$modifyFg("active","dark green")
	checkbuttonGeneralPTopes$getChild()$modifyFg("active","magenta")	
	
	if (packageVersion("gmp")< GMPOK) {
		gmpCheckButton$setActive(FALSE)
		gmpCheckButton$setSensitive(FALSE)		
	}
	
	aboutDialog <- NULL
	matrixDialog <- NULL
	
	scriptName <- "Unsaved Script"
	scriptNamed <- FALSE
	closure <- NULL
	cases <- NULL
	trace <- NULL	
	sta <- NULL	
		
	recompute.parse <- TRUE
	recompute.trace <- TRUE
	recompute.matrix <- TRUE		
	
	SCRIPT.PAGE <- 0
	RESULT.PAGE <- 1
	MATRIX.PAGE <- 2
	TRACE.PAGE <- 3		
	
	annotationTags <- list(
			resultBuffer$createTag("ann1",foreground = "blue"),			
	  		resultBuffer$createTag("ann2",foreground = "red"),
			resultBuffer$createTag("ann3",foreground = "dark green"),
	 		resultBuffer$createTag("ann4",foreground = "magenta")
	)
			
	methods <- list(
		getMainWindow = function() {
			return(mainWindow)
		},
		setScriptName = function (name="Unsaved Script") {
			scriptNamed <<- (! missing(name))
			scriptName <<- name
			mainWindow.changeTitle()
		},
		getScriptName = function() {
			return(scriptName)
		},
		isNamed = function() {
			return(scriptNamed)
		},						
		getScript = function() {
			bounds <- scriptBuffer$getBounds()
			return(scriptBuffer$getText(bounds$start,bounds$end))
		}, 
		setScript = function(script) {
			notebook$setCurrentPage(SCRIPT.PAGE)
			state$setRecomputeParse(TRUE)			
			scriptBuffer$setText(script)
			scriptBuffer$setModified(FALSE)
		},
		loadScript = function(filename) {
			content <- paste(readLines(filename,warn=FALSE),collapse="\n")
			state$setScript(content)
			state$setScriptName(filename)
			varsdialog$setVariables(c())
			checkbuttonRestrictVars$setActive(FALSE)	
		},
		cut = function() {
			scriptBuffer$cutClipboard(gtkClipboardGet("CLIPBOARD"), TRUE)		
		},
		copy = function() {
			scriptBuffer$copyClipboard(gtkClipboardGet("CLIPBOARD"))			
		},
		paste = function() {
			scriptBuffer$pasteClipboard(gtkClipboardGet("CLIPBOARD"),NULL,TRUE)
		},
		delete = function() {
			scriptBuffer$deleteSelection(TRUE, TRUE)
		},
		getModifiedAfterSave = function() {
			return(scriptBuffer$getModified())
		},		
		notifySaved=function() {
			scriptBuffer$setModified(FALSE)
		},
		setClosure = function(closure) {
			closure <<- closure			
			state$setRecomputeParse(is.null(closure))		
			if (!is.null(closure)) 
				varsdialog$setVariables(vars.call(body(state$getClosure())))										
		},
		getClosure = function() {
			return(closure)
		},
		setCases = function(cases) {			
			cases <<- cases
		},
		getCases = function() {
			return(cases)
		},
		getTagChoice = function() {
			tagComboBox$getActiveText()
		},
		getRestrictVarsFlag = function() {
			checkbuttonRestrictVars$getActive()
		},			
		setRestrictVarsFlag = function(value) {
			checkbuttonRestrictVars$setActive(value)
		},
		getPerPP = function() {
			checkbuttonPerPP$getActive()
		},
		getDebugFlag = function() {
			checkbuttonDebug$getActive()
		},
		getGraphicsFlag = function() {
			checkbuttonGraphics$getActive()
		},
		setGraphicsFlag = function(val) {
			checkbuttonGraphics$setActive(val)
			if (val) plottingview$show() else plottingview$hide()
		},
		getVariables = function() {
			if (checkbuttonRestrictVars$getActive())
				varsdialog$getVariables()
			else
				vars.call(body(state$getClosure()))
		},
		setTrace = function(trace,graphicFlag=FALSE) {			
			trace <<- trace
			state$setRecomputeTrace(FALSE)
			# the bug below is described in http://tolstoy.newcastle.edu.au/R/e17/devel/12/02/0281.html
			# we display at most 1000 lines in the trace, otherwise it is too easy to hit the bug
			nr <-  min(1000,nrow(trace$getData()))			
			tryCatch ({
				if (nr==0)
					output <- paste(capture.output(print(trace$getData())),collapse="\n")
				else
					output <- paste(capture.output(print(trace$getData()[1:nr,])),collapse="\n")
				traceBuffer$setText(output)
			    },
			    silent=TRUE, 
				error = function(c) { 			
					state$setRecomputeTrace(TRUE)
					signalCondition(analyzerCondition("Ouch! We hit a bug in the R print code. Please try again."))
					# we could try to solve ourself, but we prefer to show it is not our fault
					# output <- paste(capture.output(print(trace$getData()[1:nr,])),collapse="\n")
					# traceBuffer$setText(output)										
				}
			)					
			if(graphicFlag) plottingview$draw()
			return(TRUE)
		},
		getTrace = function() {
			return(trace)
		},	
		getManualMatrix = function() {								
			if (state$getInitializeManualMatrix()) {
				if (! script.parse()) return(NULL)
				editdialog$initializeMatrix(state$getVariables())						
				state$setInitializeManualMatrix(FALSE)	
			}			
			editdialog$getMatrix()
		},				
		getSTA = function() { 
			graphicFlag=state$getGraphicsFlag()
			if (!script.trace(graphicFlag=graphicFlag)) return(NULL)
			if (!state$getRecomputeMatrix()) return(sta)			
			trace <- state$getTrace()
		    state$setStatusMessage("Statistical Trace Analysis in progress...", TRUE)			
			if (radiobuttonPCA$getActive()) 
				sta <<- statisticalTraceAnalyzer$PCAfromTrace(trace, per.pp = state$getPerPP())
			else if (radiobuttonICA$getActive())
				sta <<- statisticalTraceAnalyzer$ICAfromTrace(trace, per.pp = state$getPerPP(), debug.trace.analyzer=checkbuttonDebug$getActive())
			else
				sta <<- StatisticalTraceAnalysis$new(trace$call, state$getManualMatrix())
			if (radiobuttonSimplifyOSCA$getActive()) {
				state$setStatusMessage("Simplification in progress...", TRUE)				
				orthogonal <- state$getOrthogonalFlag()
				theta <- state$getThreshold()
				forceint <- comboboxForceInteger$getActiveText()									
				reverse <- ! (checkbuttonReverse$getActive())
				sta <<- sta$simplifyOSCA(forceint,theta=theta,orthogonal=orthogonal,reverse=reverse)							
			} else if (radiobuttonGCDSimplification$getActive()) {
				state$setStatusMessage("GCD simplification in progress...", TRUE)
				sta <<- sta$simplifyGCD()
			}
			state$setStatusMessage("", TRUE)
			state$setRecomputeMatrix(FALSE)
			if(graphicFlag) plottingview$draw()
			sta
		},
		getSTAIfAvailable = function() {
			if (!state$getRecomputeMatrix()) 
				sta
			else
				NULL
		},
		getOrthogonalFlag = function() {
			! checkbuttonOrthogonal$getActive()
		},
		getThreshold = function() {
			thresholdAdjustment$getValue()
		},
		getLinearForms = function() {
		    strsplit(entryLinearForms$getText()," ")[[1]]
		},		
		setResult = function(results) {
			resultBuffer$setText("")
			iter <- resultBuffer$getBounds()$end
			indexes <- which(! sapply(results,is.null))						
			numlines <- min(sapply(results[indexes],length))
			
			# the line below should be equivalent to the line above
	 		# but it isn't, since it does not work
	        # iter <- resultBuffer$getEndIter()
			for (i in 1:numlines) {				
				line <- results[[(indexes[1])]][[i]]
				first <- TRUE
				if (length(grep("^[ ]*\"",line)))
					for (j in indexes) { 						
						# strips trailing spaces and double quotes					
						annotation <- sub("[ ]*$","",results[[j]][[i]])
						annotation <- gsub("\"","",annotation)
						# strips spaces at the beginning
						if (!first) annotation <- sub("^[ ]*","",annotation)
						resultBuffer$insertWithTags(iter,annotation,annotationTags[[j]])
						resultBuffer$insert(iter," ")
						first <- FALSE
					}
				else 				
					resultBuffer$insert(iter,line)									
				resultBuffer$insert(iter,"\n")
				
			}
			notebook$setCurrentPage(RESULT.PAGE)
			return(TRUE)
		},
		getGMP = function() {
			return(gmpCheckButton$getActive())
		},
		getLocalZero = function() {
			return(checkbuttonLocalzero$getActive())
		},
		selectedDomain = function(domain) {
			switch(domain,
				"intervals" = checkbuttonIntervals$getActive(),
				"parallelotopes" = checkbuttonParallelotopes$getActive(),
				"combined" = checkbuttonCombined$getActive(),
				"generalptopes" = checkbuttonGeneralPTopes$getActive(),
				FALSE
			)			
		},
		setStatusMessage = function(msg, permanent=FALSE) {	
			if (permanent) {
				statusBar$pop(1)
				if (msg!="") statusBar$push(1,msg)
			} else  {
				timeout.callback <- function() {
					statusBar$pop(0)
					return(FALSE)
				}			
				statusBar$push(0,msg)				
				statusBarTimer <- gTimeoutAdd(1000,timeout.callback)	
			}
			# Change status immediatly, in case a long action follows
			statusBar$getRootWindow()$processUpdates(TRUE)			
		},
		getRecomputeParse = function() {
			return(recompute.parse)
		},
		getRecomputeTrace = function() {
			return(recompute.trace)
		},	
		setRecomputeParse = function(flag) {
			recompute.parse <<- flag
			if (flag) {
				recompute.trace <<- TRUE
				initialize.manual <<- TRUE
				recompute.matrix <<- TRUE							
			}
		},
		setRecomputeTrace = function(flag) {
			recompute.trace <<- flag
			if (flag) {
				initialize.manual <<- TRUE
				recompute.matrix <<- TRUE				
			}
		},
		getRecomputeMatrix = function() {
			return(recompute.matrix)			
		},
		setRecomputeMatrix = function(flag, force="") {
			if (force == "manual") radiobuttonManual$setActive(TRUE)
			recompute.matrix <<- flag
		},
		setInitializeManualMatrix = function(flag) {
			initialize.manual <<- flag
		},
		getInitializeManualMatrix = function() {
			return(initialize.manual)
		}		
	)
	return(methods)
}

if (!gtkInit())
	stop("Cannot initialize GTK 2.0 library")
gtkRcParse("random.rc")

gtkSeparatorMenuItemGetType()
gtkImageMenuItemGetType()
gtkScrolledWindowGetType()
gtkTextViewGetType()
gtkStatusbarGetType()
gtkFrameGetType()
gtkTableGetType()
gtkCheckButtonGetType()
gtkHScaleGetType()
gtkComboBoxGetType()
gtkAlignmentGetType()
gtkMessageDialogGetType()
gtkAboutDialogGetType()
gtkCellRendererTextGetType()
gtkListStoreGetType()
gtkVBoxGetType()
gtkMenuBarGetType()
gtkVButtonBoxGetType()
gtkHButtonBoxGetType()
gtkRadioButtonGetType()
gtkComboBoxEntryGetType()
gtkViewportGetType()
gtkEntryGetType()

builder <- gtkBuilder()
builder$addFromFile("random.glade")

state <- state.new()

# Note that it is better to connect signal after the state
# has been created.. otherwise the state variable does not 
# exists in callbacks

builder$connectSignals(NULL)


matrixBuffer <- builder$getObject("matrixBuffer")
editdialog <- editdialog.new(state,builder)
varsdialog <- varsdialog.new(state,builder)
plottingview$init(state,builder) 

state$getMainWindow()$show()
gtkMain()
